@@ -4,27 +4,46 @@ main entry point to the program
 
 import os
 import subprocess
-import glob
 import sys
 
 import pylogconf.core
 from pytconf import register_main, config_arg_parse_and_launch, register_endpoint
 
 import github
-from pygitpub.configs import ConfigGithub, ConfigOutput
+from pygitpub.configs import ConfigGithub, ConfigOutput, ConfigAlgo
 from pygitpub.static import VERSION_STR
 from pygitpub.utils import delete
 
 
+def yield_repos():
+    g = github.Github(login_or_token=ConfigGithub.token)
+    for repo in g.get_user(ConfigGithub.username).get_repos():
+        if not ConfigAlgo.fork and repo.fork:
+            continue
+        if not ConfigAlgo.private and repo.private:
+            continue
+        if not ConfigAlgo.public and not repo.private:
+            continue
+        yield repo
+    for repo in g.get_user().get_repos(type="private"):
+        if not ConfigAlgo.fork and repo.fork:
+            continue
+        if not ConfigAlgo.private and repo.private:
+            continue
+        if not ConfigAlgo.public and not repo.private:
+            continue
+        yield repo
+
+
 @register_endpoint(
-    description="List all repos",
+    description="Set the correct website for the project on all repos where it is not correct",
     configs=[
         ConfigGithub,
+        ConfigAlgo,
     ],
 )
 def fix_website() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         if repo.homepage == "" or repo.homepage is None:
             homepage = f"{repo.html_url}"
             print(f"patching [{repo.name}]...")
@@ -32,43 +51,22 @@ def fix_website() -> None:
 
 
 @register_endpoint(
-    description="List all repos",
-    configs=[
-        ConfigGithub,
-    ],
-)
-def repos_list_verbose() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
-        if repo.description is None:
-            description = "NONE"
-        else:
-            description = repo.description
-        print(",".join([repo.name, description, str(repo.fork)]))
-
-
-@register_endpoint(
-    description="List all repos",
+    description="List repos",
     configs=[
         ConfigGithub,
+        ConfigOutput,
+        ConfigAlgo,
     ],
 )
 def repos_list() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
-        print(repo.name)
-
-
-@register_endpoint(
-    description="List all private repos",
-    configs=[
-        ConfigGithub,
-    ],
-)
-def repos_list_private() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user().get_repos(type="private"):
-        if not repo.fork:
+    for repo in yield_repos():
+        if ConfigOutput.verbose:
+            if repo.description is None:
+                description = "NONE"
+            else:
+                description = repo.description
+            print(",".join([repo.name, description, str(repo.fork)]))
+        else:
             print(f"{repo.name}")
 
 
@@ -77,11 +75,11 @@ def repos_list_private() -> None:
     configs=[
         ConfigGithub,
         ConfigOutput,
+        ConfigAlgo,
     ],
 )
 def runs_cleanup() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         for workflow in repo.get_workflows():
             existing = 0
             for run in workflow.get_runs():
@@ -107,11 +105,11 @@ def runs_cleanup() -> None:
     description="Show all runs in all workflows in all repos",
     configs=[
         ConfigGithub,
+        ConfigAlgo,
     ],
 )
 def runs_show() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         for workflow in repo.get_workflows():
             for run in workflow.get_runs():
                 print(f"{repo.name}: {workflow.name} {run.conclusion}")
@@ -121,11 +119,11 @@ def runs_show() -> None:
     description="Show all running runs in all workflows in all repositories",
     configs=[
         ConfigGithub,
+        ConfigAlgo,
     ],
 )
 def runs_show_running() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         for workflow in repo.get_workflows():
             for run in workflow.get_runs():
                 if run.conclusion is None:
@@ -136,11 +134,11 @@ def runs_show_running() -> None:
     description="Show all runs which are last and failing in all workflows and all repositories",
     configs=[
         ConfigGithub,
+        ConfigAlgo,
     ],
 )
 def runs_show_failing() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         for workflow in repo.get_workflows():
             for run in workflow.get_runs():
                 last_run = run
@@ -158,9 +156,7 @@ def runs_show_failing() -> None:
     ],
 )
 def pull_all() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    done = set()
-    for repo in g.get_user().get_repos():
+    for repo in yield_repos():
         folder = repo.name
         project = folder
         if os.path.isdir(folder):
@@ -186,25 +182,27 @@ def pull_all() -> None:
                     repo.ssh_url,
                 ]
             )
-        done.add(folder)
 
-    for gitfolder in glob.glob("*/.git"):
-        folder = os.path.split(gitfolder)[0]
-        if folder not in done:
-            project = folder
-            if not os.path.isfile(os.path.join(folder, ".skip")):
-                print(f"doing non-github project [{project}]")
-                os.chdir(folder)
-                subprocess.check_call(
-                    [
-                        "git",
-                        "pull",
-                        # '--tags',
-                    ]
-                )
-                os.chdir("..")
-            else:
-                print(f"skipping non-github project [{project}]")
+
+@register_endpoint(
+    description="Pull all projects from github",
+    configs=[
+        ConfigGithub,
+    ],
+)
+def clone_all() -> None:
+    for repo in yield_repos():
+        folder = repo.name
+        project = folder
+        if not os.path.isdir(folder):
+            print(f"project [{project}] does not exists, cloning it from [{repo.ssh_url}]...")
+            subprocess.check_call(
+                [
+                    "git",
+                    "clone",
+                    repo.ssh_url,
+                ]
+            )
 
 
 @register_endpoint(
@@ -212,11 +210,11 @@ def pull_all() -> None:
     configs=[
         ConfigGithub,
         ConfigOutput,
+        ConfigAlgo,
     ],
 )
 def workflows_run() -> None:
-    g = github.Github(login_or_token=ConfigGithub.token)
-    for repo in g.get_user(ConfigGithub.username).get_repos():
+    for repo in yield_repos():
         for workflow in repo.get_workflows():
             print(f"{repo.name}: {workflow.name}...", end='')
             sys.stdout.flush()
